from abc import ABCMeta, abstractmethod
import inspect
import weakref
import re
import holmium
from nose.tools import assert_equals, assert_true

class FacetMeta(type):
    __metaclass__ = ABCMeta

    def __new__(cls, name, bases, attrs):
        _t = super(FacetMeta, cls).__new__(cls, name, bases, attrs)
        extra_doc = """
    :param bool debug: if True a failure to evaluate will not result in an exception, only a log warning
    :param bool required: if False a failure to evaluate will be treated as a noop.
"""
        _t.__doc__ = (_t.__doc__ or "") + extra_doc
        return _t


class Facet(object):
    """
    base class to implement an attribute of a page
    """
    __ARGS__ = []
    __OPTIONS__ = {}
    __metaclass__ = FacetMeta

    def __init__(self, required=True, debug=False, **kwargs):
        self.arguments = {}
        self.options = {}
        self.required = required
        self.debug = debug
        self._parent_class = None
        for arg in self.__ARGS__:
            if arg not in kwargs.keys():
                raise AttributeError("%s is a required argument for %s" % (
                arg, self.__class__.__name__))
            else:
                self.arguments[arg] = kwargs[arg]
                kwargs.pop(arg)
        for arg in self.__OPTIONS__:
            if arg in kwargs.keys():
                self.options[arg] = kwargs[arg]
                kwargs.pop(arg)
            else:
                self.options[arg] = self.__OPTIONS__[arg]
        if kwargs:
            raise AttributeError("unknown argument(s) to %s (%s)" % (
            self.__class__.__name__, ",".join(kwargs.keys())))

    def register(self, obj):
        """
        registers a :class:`Facet` on an object

        :param holmium.core.facets.Faceted obj: the object to register the facet on.
        """
        if inspect.isclass(obj):
            obj.get_class_facets().append(self)
            self.parent_class = obj
        else:
            obj.get_instance_facets().append(self)
            self.parent_class = obj.__class__


    def __call__(self, obj):
        self.register(obj)
        return obj

    def get_name(self):
        return self.__class__.__name__

    @property
    def parent_class(self):
        return self._parent_class()

    @parent_class.setter
    def parent_class(self, parent):
        self._parent_class = weakref.ref(parent)

    def get_parent_name(self):
        return (self.parent_class and self.parent_class.__name__) or None

    @abstractmethod
    def evaluate(self, driver):
        """
        evaluate whether this facet holds true. Raise an Exception
        if not.

        :param selenium.webdriver.remote.webdriver.WebDriver driver: the webdriver
        """
        raise NotImplementedError


class FacetError(Exception):
    """
    exception raised when a facet has an error
    or can't complete

    :param holmium.core.facets.Facet facet: the facet that failed to evaluate
    :param exceptions.Exception exc: the inner exception that caused the failure
    """

    def __init__(self, facet, exc=None):
        self.message = "%s failed to exhibit facet %s" % (
            facet.get_parent_name(), facet.get_name())
        if exc:
            self.message += " with error %s" % exc
        super(FacetError, self).__init__(self.message)


class FacetCollection(list):
    """
    utility collection class for pageobjects to encapsulate
    facets
    """

    def __init__(self):
        super(FacetCollection, self).__init__()


    def evaluate_all(self, driver):
        """
        iterate over all registered :class:`Facet` objects and validate them

        :param selenium.webdriver.remote.webdriver.WebDriver driver: the webdriver
        """
        for facet in self:
            try:
                facet.evaluate(driver)
            except Exception, e:
                if facet.debug:
                    holmium.core.log.warn(FacetError(facet, e))
                elif facet.required:
                    raise FacetError(facet, e)


class Faceted(object):
    """
    mixin for objects that want to have facets registered
    on them.
    """

    def __init__(self):
        self.instance_facets = FacetCollection()
        super(Faceted, self).__init__()

    @classmethod
    def get_class_facets(cls):
        if not hasattr(cls, "class_facets"):
            cls.class_facets = FacetCollection()
        return cls.class_facets

    def get_instance_facets(self):
        return object.__getattribute__(self, "instance_facets")


    def evaluate(self):
        import holmium.core

        safe_get = lambda e: object.__getattribute__(self, e)
        driver = holmium.core.Page.get_driver()
        instance_facets = safe_get("get_instance_facets")()
        class_facets = safe_get("get_class_facets")()

        class_facets.evaluate_all(driver)
        instance_facets.evaluate_all(driver)


class defer(Facet):
    """
    :param holmium.core.Page page: the page object that is expected to be deferred to
    :param function action: a callable that takes the page object instance as the first argument
    :param dict action_arguments: (optional) dictionary of arguments to pass to `action`
    """
    __ARGS__ = ["page", "action"]
    __OPTIONS__ = {"action_arguments": {}}

    def evaluate(self, driver):
        page_cls = self.arguments["page"]
        page = page_cls(driver)
        return self.arguments["action"](page,
                                        **self.options["action_arguments"]
        )


class title(Facet):
    """
    enforces the title of the current page.

    :param str title: a regular expression to match the title.
    """
    __ARGS__ = ["title"]

    def evaluate(self, driver):
        assert_true(re.compile(self.arguments["title"]).match(driver.title),
                    "title did not match %s" % self.arguments["title"]
        )



class cookie(Facet):
    """
    enforces the existence (and optionally the value) of a cookie.

    :param str name: name of the cookie
    :param dict value: (optional) dict (or callable) to validate the value of the cookie.

    """
    __ARGS__ = ["name"]
    __OPTIONS__ = {"value": None}

    def evaluate(self, driver):
        cookie_value = driver.get_cookie(self.arguments["name"])
        if cookie_value and self.options["value"]:
            if callable(self.options["value"]):
                assert self.options["value"](cookie_value)
            else:
                assert_equals( cookie_value , self.options["value"] )
        else:
            assert_true(cookie_value != None, "cookie %s does not exist" % self.arguments["name"])


class strict(Facet):
    """
    enforces that every element declared in the :class:`Page` or :class:`Section`
    be present.
    """

    def evaluate(self, driver):
        pass

    def __call__(self, obj):
        import holmium.core.pageobject

        for element in inspect.getmembers(obj):
            if isinstance(element[1], holmium.core.pageobject.ElementGetter):
                element[1].is_facet = True
                element[1].is_debug_facet = self.debug
        return obj


class ElementFacet(Facet):
    """
    utility trait used when validating an
    :class:`holmium.core.pageobject.ElementGetter` subclass
    """

    def __init__(self, element, element_name, **kwargs):
        self.element_name = element_name
        self.element = element
        super(ElementFacet, self).__init__(required=True, **kwargs)

    def evaluate(self, driver):
        assert_true (self.element.__get__(self.parent_class,
                                            self.parent_class), "No such element")

    def get_name(self):
        return self.element_name



import inspect
import weakref
import types
import threading
import contextlib
import collections
from functools import wraps

import selenium.webdriver.common.by
from selenium.common.exceptions import NoSuchElementException, TimeoutException, NoSuchFrameException
from selenium.webdriver.support.ui import WebDriverWait, Select
from selenium.webdriver.remote.webelement import WebElement
from ordereddict import OrderedDict

import holmium

from holmium.core.facets import Faceted, ElementFacet


class Locators(selenium.webdriver.common.by.By):
    """
    proxy class to access locator types
    """
    pass


def enhanced(web_element):
    """
    incase a higher level abstraction for a WebElement is available
    we will use that in Pages. (e.g. a select element is converted into
    :class:`selenium.webdriver.support.ui.Select`)
    """
    abstraction_mapping = {'select': Select}
    if web_element.tag_name in abstraction_mapping.keys():
        return abstraction_mapping[web_element.tag_name](web_element)
    return web_element


class ElementList(list):
    """
    proxy to a standard list which would be stored in
    a :class:`holmium.core.Page`.
    """

    def __init__(self, instance, *args, **kwargs):
        self.instance = weakref.ref(instance)
        list.__init__(self, *args, **kwargs)

    def __getitem__(self, index):
        return list.__getitem__(self, index).__get__(self.instance(),
                                                     self.instance().__class__)


class ElementDict(dict):
    """
    proxy to a standard dict which would be stored in
    a :class:`holmium.core.Page`.
    """

    def __init__(self, instance, *args, **kwargs):
        self.instance = weakref.ref(instance)
        dict.__init__(self, *args, **kwargs)

    def __getitem__(self, key):
        return dict.__getitem__(self, key).__get__(self.instance(),
                                                   self.instance().__class__)

class Registry(type):
    """
    simple meta class to keep track of all page objects registered
    """
    pages = {}
    def __new__(cls, *args, **kwargs):
        page = super(Registry, cls).__new__(cls, *args, **kwargs)
        Registry.pages[args[0]] = page
        return page



class Page(Faceted):
    """
    Base class for all page objects to extend from.
    void Instance methods implemented by subclasses are provisioned
    with fluent wrappers to facilitate with writing code such as::

        class Google(Page):
            def enter_query(self):
                ....

            def submit_search(self):
                ....

            def get_results(self):
                ....

        assert len(Google().enter_query("page objects").submit_search().get_results()) > 0

    """
    __metaclass__ = Registry
    local = threading.local()

    def __init__(self, driver, url=None, iframe=None):
        super(Page, self).__init__()
        self.driver = driver
        self.touched = False
        self.initialized = False
        if url:
            self.home = url
        elif driver.current_url:
            self.home = driver.current_url
        else:
            self.home = None

        self.iframe = iframe
        for el in inspect.getmembers(self.__class__):
            def update_element(el, name):
                if issubclass(el.__class__, ElementGetter):
                    el.iframe = self.iframe
                    if el.is_facet:
                        facet = ElementFacet(el, name, debug=el.is_debug_facet)
                        facet.register(self)

            if issubclass(el[1].__class__, list):
                for item in el[1]:
                    update_element(item, el[0])
                self.__setattr__(el[0], ElementList(self, el[1]))
            elif issubclass(el[1].__class__, dict):
                for item in el[1].values():
                    update_element(item, el[0])
                self.__setattr__(el[0], ElementDict(self, el[1]))
            else:
                update_element(el[1], el[0])

        if url:
            self.driver.get(url)
        self.initialized = True

    @contextlib.contextmanager
    def scope(self):
        Page.local.driver = object.__getattribute__(self, "driver")
        yield

    @staticmethod
    def get_driver():
        return Page.local.driver

    def go_home(self):
        """
        returns the page object to the url it was initialized with
        """
        self.driver.get(self.home)

    def __getattribute__(self, key):
        """
        to enable fluent access to page objects, instance methods that
        don't return a value, instead return the page object instance.
        """

        attr_getter = lambda key: object.__getattribute__(self, key)
        attr_setter = lambda key, value: object.__setattr__(self, key, value)

        with attr_getter("scope")():
            if not attr_getter("touched") and attr_getter("initialized"):
                driver = attr_getter("driver")
                attr_getter("evaluate")()
                attr_setter("touched", True)
            attr = attr_getter(key)
            # check if home url is set, else update.
            if not attr_getter("home"):
                holmium.core.log.debug(
                    "home url not set, attempting to update.")
                attr_setter("home", attr_getter("driver").current_url)

            if isinstance(attr, types.MethodType):
                @wraps(attr)
                def wrap(*args, **kwargs):
                    resp = attr(*args, **kwargs)
                    if None == resp:
                        holmium.core.log.debug(
                            "method %s returned None, using fluent response" % attr.func_name)
                        resp = self
                    return resp

                return wrap
            return attr



class ElementGetter(object):
    """
    internal class to encapsulate the logic used by
    :class:`holmium.core.Element`
    & :class:`holmium.core.Elements`
    """

    def __init__(self, locator_type,
                 query_string,
                 base_element=None,
                 timeout=1,
                 value=lambda el: el,
                 only_if=lambda el: el != None,
                 facet=False):
        """
        :param holmium.core.Locators locator_type: selenium locator to use when locating the element
        :param str query_string: the value to pass to the locator
        :param holmium.core.Element base_element: a reference to another element under which to locate this element.
        :param int timeout: time to implicitely wait for the element
        :param lambda value: transform function for the value of the element.
         The located :class:`selenium.webdriver.remote.webelement.WebElement`
         instance is passed as the only argument to the function.
        :param function only_if: extra validation function that is called repeatedly upto :attr:`timeout`
         after the element is located transform function for the value of the element.
         The located :class:`selenium.webdriver.remote.webelement.WebElement`
         instance is passed as the only argument to the function.
        :param bool facet: flag to  treat this element as a facet.
        """
        self.query_string = query_string
        self.locator_type = locator_type
        self.timeout = timeout
        self.driver = None
        self.iframe = None
        self.base_element = base_element
        self.value_mapper = value
        self.root_fn = lambda: Page.get_driver()
        self.only_if = only_if
        holmium.core.log.debug("locator:%s, query_string:%s, timeout:%d" %
                               (locator_type, query_string, timeout))
        self.is_facet = facet
        self.is_debug_facet = False

    @property
    def root(self):
        return self.root_fn()

    @root.setter
    def root(self, root_fn):
        self.root_fn = root_fn

    def get_element(self, method=None):

        if self.base_element:
            if isinstance(self.base_element, types.LambdaType):
                el = self.base_element()
                _meth = getattr(el, method.im_func.func_name)
            elif isinstance(self.base_element, Element):
                _meth = getattr(self.base_element.__get__(self, self.__class__),
                                method.im_func.func_name)
            elif isinstance(self.base_element, WebElement):
                _meth = getattr(self.base_element, "find_element")
            else:
                raise TypeError("invalid base_element type (%s) used" % (
                    type(self.base_element)))
        else:
            _meth = method
        holmium.core.log.debug(
            "looking up locator:%s, query_string:%s, timeout:%d" %
            (self.locator_type, self.query_string, self.timeout))

        if self.iframe:
            Page.local.driver.switch_to_default_content()
            Page.local.driver.switch_to_frame(self.iframe)

        if self.timeout:
            try:
                callback = lambda _: _meth(self.locator_type, self.query_string) \
                    and self.only_if( _meth(self.locator_type, self.query_string))
                WebDriverWait(self.root, self.timeout).until(callback)
            except TimeoutException:
                holmium.core.log.debug(
                    "unable to find element %s after waiting for %d seconds" % (
                        self.query_string, self.timeout))
                raise
        return _meth(self.locator_type, self.query_string)


class Element(ElementGetter):
    """
    Utility class to get a :class:`selenium.webdriver.remote.webelement.WebElement`
    by querying via one of :class:`holmium.core.Locators`

    :param holmium.core.Locators locator_type: selenium locator to use when locating the element
    :param str query_string: the value to pass to the locator
    :param holmium.core.Element base_element: a reference to another element under which to locate this element.
    :param int timeout: time to implicitely wait for the element
    :param lambda value: transform function for the value of the element.
     The located :class:`selenium.webdriver.remote.webelement.WebElement` instance is passed as the only argument to the function.
    :param function only_if: extra validation function that is called repeatedly upto :attr:`timeout`
     after the element is located transform function for the value of the element.
     The located :class:`selenium.webdriver.remote.webelement.WebElement`
     instance is passed as the only argument to the function.
    :param bool facet: flag to  treat this element as a facet.
    """

    def __get__(self, instance, owner):
        if not instance:
            return self
        try:
            return self.value_mapper(
                enhanced(self.get_element(self.root.find_element))
            ) if self.root else None
        except (NoSuchElementException, TimeoutException):
            return None


class Elements(ElementGetter):
    """
    Utility class to get a collection of :class:`selenium.webdriver.remote.webelement.WebElement`
    objects by querying via one of :class:`holmium.core.Locators`

    :param holmium.core.Locators locator_type: selenium locator to use when locating the element
    :param str query_string: the value to pass to the locator
    :param holmium.core.Element base_element: a reference to another element under which to locate this element.
    :param int timeout: time to implicitely wait for the element
    :param lambda value: transform function for each element in the collection.
     The located :class:`selenium.webdriver.remote.webelement.WebElement` instance is passed as the only argument to the function.
    :param function only_if: extra validation function that is called repeatedly upto :attr:`timeout`
     after the element is located transform function for the value of the element.
     The located :class:`selenium.webdriver.remote.webelement.WebElement`
     instance is passed as the only argument to the function.
    :param bool facet: flag to  treat this element as a facet.
    """

    def __getitem__(self, idx):
        return lambda: self.__get__(self, self.__class__)[idx]

    def __get__(self, instance, owner):
        if not instance:
            return self
        try:
            return [self.value_mapper(enhanced(el)) for el in
                self.get_element(self.root.find_elements)] if self.root else []
        except (NoSuchElementException, TimeoutException):
            return []

class ElementMap(Elements):
    """
    Used to create dynamic dictionaries based on an element locator specified by one of
    :class:`holmium.core.Locators`.

    The wrapped dictionary is an :class:`collections.OrderedDict` instance.

    :param holmium.core.Locators locator_type: selenium locator to use when locating the element
    :param str query_string: the value to pass to the locator
    :param holmium.core.Element base_element: a reference to another element under which to locate this element.
    :param int timeout: time to implicitely wait for the element
    :param bool facet: flag to  treat this element as a facet.
    :param lambda key: transform function for mapping a key to a WebElement in the collection.
     The located :class:`selenium.webdriver.remote.webelement.WebElement` instance is passed as the only argument to the function.
    :param lambda value: transform function for the value when accessed via the key.
     The located :class:`selenium.webdriver.remote.webelement.WebElement` instance is passed as the only argument to the function.
    :param function only_if: extra validation function that is called repeatedly upto :attr:`timeout`
     after the element is located transform function for the value of the element.
     The located :class:`selenium.webdriver.remote.webelement.WebElement`
     instance is passed as the only argument to the function.
    """

    def __init__(self, locator_type,
                 query_string=None,
                 base_element=None,
                 timeout=1,
                 key=lambda el: el.text,
                 value=lambda el: el,
                 only_if=lambda el: el != None,
                 facet=False):
        super(ElementMap, self).__init__(locator_type, query_string,
                                         base_element,
                                         timeout, facet)
        self.key_mapper = key
        self.value_mapper = value

    def __get__(self, instance, owner):
        if not instance:
            return self
        try:
            return OrderedDict(
            (self.key_mapper(el), self.value_mapper(enhanced(el))) for el in
            self.get_element(self.root.find_elements)) if self.root else {}
        except (NoSuchElementException, TimeoutException):
            return {}

    def __getitem__(self, key):
        return lambda: self.__get__(self, self.__class__)[key]


class Section(Faceted):
    """
    Base class to encapsulate reusable page sections::

        class MySection(Section):
            things = Elements( .... )

        class MyPage(Page):
            section_1 =  MySection(Locators.CLASS_NAME, "section")
            section_2 =  MySection(Locators.ID, "unique_section")

    """

    def __init__(self, locator_type, query_string, iframe=None):

        super(Section, self).__init__()
        self.touched = False
        self.locator_type = locator_type
        self.query_string = query_string
        self.iframe = iframe
        self.__root_val = None
        self.element_members = {}
        for element in inspect.getmembers(self.__class__):
            if issubclass(element[1].__class__, ElementGetter):
                self.element_members[element[0]] = element[1]
                if element[1].is_facet:
                    facet = ElementFacet(element[1], element[0], debug=element[1].is_debug_facet)
                    facet.register(self)

    def __get__(self, instance, owner):
        for element in self.element_members.values():
            element.root = lambda: self.root
        return self

    def __getattribute__(self, item):
        attr_getter = lambda key: super(Section, self).__getattribute__(key)
        attr_setter = lambda key, value: super(Section, self).__setattr__(key,
                                                                          value)
        members = attr_getter("element_members")
        touched = attr_getter("touched")
        if not touched and item in members:
            attr_getter("evaluate")()
            attr_setter("touched", True)
        return attr_getter(item)


    @property
    def root(self):
        if self.iframe:
            try:
                Page.get_driver().switch_to_default_content()
                Page.get_driver().switch_to_frame(self.iframe)
            except NoSuchFrameException:
                holmium.core.log.error(
                    "unable to switch to iframe %s" % self.iframe)
        try:
            return self.__root_val or Page.get_driver().find_element(
                self.locator_type, self.query_string
            )
        except (NoSuchElementException, TimeoutException):
            return None

    @root.setter
    def root(self, val):
        self.__root_val = val


class Sections(Section, collections.Sequence):
    """
    Base class for an Iterable view of a collection of :class:`holmium.core.Section`
    objects.
    """

    def __init__(self, locator_type, query_string, iframe=None, timeout=0):
        super(Sections, self).__init__(locator_type, query_string, iframe)
        self.timeout = timeout

    def __getelements__(self):
        if self.timeout:
            try:
                WebDriverWait(Page.get_driver(), self.timeout).until(
                    lambda _: Page.get_driver().find_elements(self.locator_type, self.query_string))
            except TimeoutException:
                holmium.core.log.debug(
                    "unable to find element %s after waiting for %d seconds" % (
                        self.query_string, self.timeout))
        return Page.get_driver().find_elements(self.locator_type,
                                               self.query_string)

    def __iter__(self):
        for element in self.__getelements__():
            self.root = element
            yield self

    def __len__(self):
        return len(self.__getelements__())

    def __getitem__(self, idx):
        _idx = 0
        for item in self:
            if idx == _idx:
                break
            _idx += 1
            if idx > _idx:
                raise IndexError("Sections index (%d) out of range" % idx)
        if len(self) == 0:
            raise IndexError("Sections index (%d) out of range" % idx)
        return self



from holmium.core.logger import log
from holmium.core.testcase import TestCase
from holmium.core.pageobject import Locators, Page, Section, Sections, Element, Elements
from holmium.core.pageobject import ElementMap
from holmium.core.noseplugin import HolmiumNose
from holmium.core.deprecated import PageObject, PageElement, PageElements, PageElementMap, HolmiumTestCase
from holmium.core.config import Config



import imp
import os
import json
from nose.plugins.base import Plugin
from nose.plugins.skip import SkipTest
import sys
import holmium.core
from holmium.core.config import HolmiumConfig, configure
try:
    from fresher import ftc
except ImportError:
    ftc = None

class HolmiumNose(Plugin):
    """
    nose plugin to allow bootstrapping testcases with a selenium driver
    """
    name = "holmium"
    enabled = False


    def __init__(self):
        Plugin.__init__(self)
        self.driver = None
        self.config = {}
        self.environment = None
        self.driver_initializer_fn = lambda _: None
        self.logger = holmium.core.log

    def options(self, parser, env):
        """
        Register command line options
        """
        parser.add_option("", "--with-holmium", dest="ho_enabled",
                          action="store_true",
                          help="use holmium to setup test case environment")
        parser.add_option("", "--holmium-environment", dest="ho_env",
                          help="environment to pass to holmium test case configuration")
        parser.add_option("", "--holmium-browser", dest="ho_browser",
                          type="choice",
                          choices=list(
                              holmium.core.config.browser_mapping.keys()),
                          help="the selenium driver to invoke")
        parser.add_option("", "--holmium-remote", dest="ho_remote",
                          help="full url to remote selenium instance")
        parser.add_option("", "--holmium-capabilities", dest="ho_cap",
                          help="json dictionary of extra capabilities")
        parser.add_option("", "--holmium-useragent", dest="ho_ua",
                          help="User-agent string to use. Only available for firefox & chrome")
        parser.add_option("", "--holmium-ignore-ssl-errors",
                          action="store_true", dest="ho_ignore_ssl",
                          help="Ignore ssl errors while running tests")

    def configure(self, options, conf):
        if options.ho_enabled:
            browser = options.ho_browser or os.environ.get("HO_BROWSER", "")
            environment = options.ho_env or os.environ.get("HO_ENVIRONMENT", "")
            remote_url = options.ho_remote or os.environ.get("HO_REMOTE", "")
            user_agent = options.ho_ua or os.environ.get("HO_USERAGENT", "")
            ignore_ssl = options.ho_ignore_ssl or os.environ.get(
                "HO_IGNORE_SSL_ERRORS", False)
            caps = options.ho_cap and json.loads(options.ho_cap) or {}

            self.holmium_config = holmium_config = HolmiumConfig(browser,
                                                                 remote_url,
                                                                 caps,
                                                                 user_agent,
                                                                 environment,
                                                                 ignore_ssl)

            args = configure(holmium_config)
            if holmium_config.remote:
                driver = holmium.core.config.browser_mapping["remote"]
            else:
                driver = holmium.core.config.browser_mapping[
                    holmium_config.browser]

            self.driver_initializer_fn = lambda: driver(**args)
            self.enabled = True

    def beforeTest(self, test):
        try:
            if not self.driver:
                self.driver = self.driver_initializer_fn()
        except Exception as e:
            self.logger.exception("failed to initialize selenium driver")
            raise SkipTest(
                "holmium could not be initialized due to a problem with the required selenium driver")

        base_file = test.address()[0] if not hasattr(test.test,
                                                     "feature") else test.test.feature.src_file
        config_path = os.path.join(os.path.split(base_file)[0], "config")
        try:
            config = None
            if os.path.isfile(config_path + ".json"):
                config = json.loads(open(config_path + ".json").read())
            elif os.path.isfile(config_path + ".py"):
                if "holmium_testcase_config" in sys.modules:
                    del sys.modules["holmium_testcase_config"]
                config = imp.load_source("holmium_testcase_config", config_path + ".py").config
            if config:
                self.config = holmium.core.Config(config, {
                "holmium": self.holmium_config})
        except Exception as e:
            self.logger.debug("unable to load %s" % config_path)
            raise SkipTest(
                "error in loading config file at path %s" % config_path)
        if HolmiumNose.is_freshen_test(test) and ftc:
            ftc.config = self.config
        else:
            setattr(test.test, "config", self.config)

    def startTest(self, test):
        if self.driver:
            if HolmiumNose.is_freshen_test(test) and ftc:
                ftc.driver = self.driver
            else:
                setattr(test.test, "driver", self.driver)
            self.driver.delete_all_cookies()

    def finalize(self, result):
        if self.driver:
            self.driver.quit()

    @staticmethod
    def is_freshen_test(test):
        return test.address()[1] == "fresher.noseplugin"



import logging
#: shared stream logger for all holmium classes
log = logging.Logger("holmium.core")
log.setLevel(logging.INFO)
shandler = logging.StreamHandler()
format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
formatter = logging.Formatter(format)
shandler.setFormatter(formatter)
log.addHandler(shandler)



# -*- coding: utf-8 -*-
import unittest
import threading
import os

import mock

import holmium.core


support = os.path.join(os.path.dirname(__file__), "support")


class BugReports(unittest.TestCase):
    def test_multiple_pageinstances(self):
        """ https://github.com/alisaifee/holmium.core/issues/4
        """
        class p(holmium.core.Page):
            el = holmium.core.Element(holmium.core.Locators.NAME, "name")
        d1,d2=mock.Mock(), mock.Mock()
        p1,p2=p(d1, "http://p1"),p(d2, "http://p2")
        e1,e2=mock.Mock(), mock.Mock()
        e2.tag_name = e1.tag_name = "div"
        e1.text = "t1"
        e2.text = "t2"
        d1.find_element.return_value = e1
        d2.find_element.return_value = e2
        self.assertEquals(p1.el.text,"t1")
        self.assertEquals(p2.el.text,"t2")
        self.assertEquals(d1.get.call_count, 1)
        self.assertEquals(d2.get.call_count, 1)
    def test_multiple_pageinstances_multithreaded(self):
        """ https://github.com/alisaifee/holmium.core/issues/4
        """
        class p(holmium.core.Page):
            el = holmium.core.Element(holmium.core.Locators.NAME, "name")
        class p2(holmium.core.Page):
            el = holmium.core.Element(holmium.core.Locators.NAME, "name")

        def exec_page_in_thread(p):
            p.go_home()
            self.assertEquals(p.t, p.el.text)
            self.assertEquals(p.driver.get.call_count, 2)

        def build_pages(po):
            pages=[]
            for i in range(0,100):
                d = mock.Mock()
                e = mock.Mock()
                e.text = str(p)+str(i)
                e.tag = "div"
                d.find_element.return_value = e
                _p = po(d,"http://%s" % i)
                _p.t = str(p)+str(i)
                pages.append(_p)
            return pages
        pages = build_pages(p)
        pages.extend(build_pages(p2))

        threads = [threading.Thread(target=exec_page_in_thread, args=(p,)) for p in pages]
        [k.start() for k in threads]
        [k.join() for k in threads]

    def test_fluent_response(self):
        """ https://github.com/alisaifee/holmium.core/issues/8
        """
        class p(holmium.core.Page):
            el = holmium.core.Element(holmium.core.Locators.NAME, "name")
            def f1(self):
                self.el.click()
            def f2(self):
                return False
            def f3(self):
                return []
            def f4(self):
                return {}
            def f5(self):
                return ""
        d1=mock.Mock()
        p1=p(d1, "http://p1")
        self.assertEquals(p1, p1.f1())
        self.assertEquals(False, p1.f2())
        self.assertEquals([], p1.f3())
        self.assertEquals({}, p1.f4())
        self.assertEquals("",p1.f5())




import types
import warnings
import pageobject
import testcase


class Deprecated(object):
    def __new__(cls, *args, **kwargs):
        raise SyntaxError("%s has been removed as of version 0.4. Use %s instead" % (
            cls.cur, cls.alt.__name__
            )
        )


def construct_deprecated(name, alt):
    doc = """Deprecated alias for :class:`%s`""" % alt.__name__
    cls = types.ClassType(name, (Deprecated, alt),
                          dict(cur=name, alt=alt, __doc__=doc))
    return cls


PageObject = construct_deprecated("PageObject", pageobject.Page)
PageElement = construct_deprecated("PageElement", pageobject.Element)
PageElements = construct_deprecated("PageElements", pageobject.Elements)
PageElementMap = construct_deprecated("PageElementMap", pageobject.ElementMap)
HolmiumTestCase = construct_deprecated("HolmiumTestCase", testcase.TestCase)

warnings.simplefilter("always")



import unittest
import os

import mock

import holmium.core
import holmium.core.config


def runtc(env, validations):
    try:
        _pre_env = dict(os.environ)
        os.environ.update(env)
        class t(holmium.core.TestCase):
            def runTest(self):
                for validation in validations:
                    self.assertTrue(validation(self))

        test = t()
        test.setUpClass()
        test.setUp()
        test.runTest()
        test.tearDown()
        test.tearDownClass()
    finally:
        os.environ = _pre_env

def build_mock_mapping(name):
    mock_driver = mock.Mock()
    browser_mapping = {name:mock_driver}
    mock_driver.return_value.name = name
    return browser_mapping

class TestCaseTests(unittest.TestCase):
    def test_set_browser(self):
        with mock.patch.dict('holmium.core.config.browser_mapping', build_mock_mapping("phantomjs")):
            runtc({"HO_BROWSER":"phantomjs"}, [lambda s:s.driver!=None])

    def test_set_browser_phantom(self):
        with mock.patch.dict('holmium.core.config.browser_mapping', build_mock_mapping("phantomjs")):
            runtc({"HO_BROWSER":"phantomjs"}, [lambda s:s.driver!=None, lambda s:s.driver.name == "phantomjs"])

    def test_set_browser_remote(self):
        with mock.patch.dict('holmium.core.config.browser_mapping', build_mock_mapping("remote")):
            runtc({"HO_BROWSER":"phantomjs", "HO_REMOTE":"http://lala.com"}, [lambda s:s.driver!=None, lambda s:s.driver.name == "remote"])

    def test_auto_config_json(self):
        with mock.patch("os.path.isfile") as isfile:
            with mock.patch.dict('holmium.core.config.browser_mapping', build_mock_mapping("phantomjs")):
                isfile.side_effect = lambda name:name.find("json")>0
                with mock.patch("__builtin__.open") as op:
                    op.return_value.read.return_value = '{"default": {"test": 1}}'
                    runtc({"HO_BROWSER":"phantomjs"}, [lambda s:s.config["test"]==1])

    def test_auto_config_py(self):
        with mock.patch("os.path.isfile") as isfile:
            with mock.patch.dict('holmium.core.config.browser_mapping', build_mock_mapping("phantomjs")):
                isfile.side_effect = lambda name:name.find("py")>0
                with mock.patch("imp.load_source") as load_source:
                    m_config = mock.Mock()
                    m_config.config = {"default":{"test":1}}
                    load_source.return_value = m_config
                    runtc({"HO_BROWSER":"phantomjs"}, [lambda s:s.config["test"]==1])

    def test_set_useragent(self):
        with mock.patch.dict('holmium.core.config.browser_mapping', build_mock_mapping("firefox")):
            runtc({"HO_BROWSER":"firefox", "HO_USERAGENT":"holmium.core"}, [])
            call_args = holmium.core.config.browser_mapping["firefox"].call_args
            assert len(call_args) == 2
            assert call_args[1].has_key("firefox_profile")
            ff_profile = call_args[1]["firefox_profile"]
            ua = ff_profile.default_preferences["general.useragent.override"]
            self.assertEquals(ua, '"holmium.core"')



import unittest
import holmium.core
from tests.utils import get_driver, make_temp_page


class ElementMapTest(unittest.TestCase):
    def setUp(self):
        self.driver = get_driver()

    def test_basic_element(self):
        class SimplePage(holmium.core.Page):
            el_map_default = holmium.core.ElementMap(holmium.core.Locators.CLASS_NAME, "simple_class")
            el_map_keymapper = holmium.core.ElementMap(holmium.core.Locators.CLASS_NAME, "simple_class"
                                                            , key = lambda el: el.find_element_by_tag_name("a").text)
            el_map_valuemapper = holmium.core.ElementMap(holmium.core.Locators.CLASS_NAME, "simple_class"
                                                            , value = lambda el: el.find_element_by_tag_name("a").text)
            el_map_keyvaluemapper = holmium.core.ElementMap(holmium.core.Locators.CLASS_NAME, "simple_class"
                                                            , key = lambda el: el.find_element_by_tag_name("a").text
                                                            , value = lambda el: el.find_element_by_tag_name("a").get_attribute("href"))


        uri = make_temp_page("""
            <body>
                <div class="simple_class">
                    simple class el 1
                    <a href="http://el1.com/">element 1</a>
                </div>
                <div class="simple_class">
                    simple class el 2
                    <a href="http://el2.com/">element 2</a>
                </div>
                <div class="simple_class">
                    simple class el 3
                    <a href="http://el3.com/">element 3</a>
                </div>
            </body>
        """)
        page = SimplePage(self.driver, uri)
        self.assertEquals(page.el_map_default.keys() , ["simple class el 1 element 1", "simple class el 2 element 2", "simple class el 3 element 3"] )
        self.assertEquals(page.el_map_keymapper.keys() , ["element 1", "element 2", "element 3"] )
        self.assertEquals(page.el_map_valuemapper.values() , ["element 1", "element 2", "element 3"] )
        self.assertEquals(page.el_map_keyvaluemapper.keys() , ["element 1", "element 2", "element 3"] )
        self.assertEquals(page.el_map_keyvaluemapper.values() , ["http://el1.com/", u"http://el2.com/", u"http://el3.com/"])




import os
import unittest
import inspect
import imp
import json

import holmium
from holmium.core.config import HolmiumConfig, configure


class TestCase(unittest.TestCase):
    """
    Base class for creating test classes for writing holmium driven
    test cases. More details can be found at :ref:`testing-unittest`.
    """

    @classmethod
    def setUpClass(cls):
        """
        create the driver and configure it before any of the tests run.
        """
        cls.driver = None
        base_file = inspect.getfile(cls)
        config_path = os.path.join(os.path.split(base_file)[0], "config")
        browser = os.environ.get("HO_BROWSER", "firefox")
        user_agent = os.environ.get("HO_USERAGENT", "")
        remote = os.environ.get("HO_REMOTE", None)
        environment = os.environ.get("HO_ENV", "development")
        ignore_ssl = os.environ.get("HO_IGNORE_SSL_ERRORS", False)
        holmium_config = HolmiumConfig(browser, remote, {}, user_agent,
                                       environment, ignore_ssl)
        args = configure(holmium_config)
        config = None
        if os.path.isfile(config_path + ".json"):
            config = json.loads(open(config_path + ".json").read())
        elif os.path.isfile(config_path + ".py"):
            config = imp.load_source("config", config_path + ".py").config
        if config:
            cls.config = holmium.core.Config(config,
                                             {"holmium": holmium_config})
        if remote:
            driver = holmium.core.config.browser_mapping["remote"]
        else:
            driver = holmium.core.config.browser_mapping[holmium_config.browser]
        cls.driver = driver(**args)
        super(TestCase, cls).setUpClass()

    @classmethod
    def tearDownClass(cls):
        """
        quit the driver after all the test methods in the class have
        finished.
        """
        if cls.driver:
            cls.driver.quit()
        super(TestCase, cls).tearDownClass()

    def tearDown(self):
        """
        clear the cookies on the driver after each test
        """
        if hasattr(self, "driver"):
            self.driver.delete_all_cookies()
        super(TestCase,self).tearDown()
